﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace ArduSpdif
{
    internal class Program
    {
        static int Main(string[] args)
        {
            if (args.Length != 2)
            {
                Console.WriteLine("2 Argumente angeben : 1. Quellfilename; 2. Zielfilename");
                return 42;
            }
            if (String.IsNullOrWhiteSpace(args[0]) || String.IsNullOrWhiteSpace(args[1]))
            {
                Console.WriteLine("2 Argumente angeben : 1. Quellfilename; 2. Zielfilename");
                return 42;
            }
            try
            {
                string sfn = Path.GetFullPath(args[0].Trim());
                string tfn = Path.GetFullPath(args[1].Trim());
                if (sfn.ToLowerInvariant() == tfn.ToLowerInvariant())
                {
                    Console.Error.WriteLine("Quelle und Ziel dürfen nicht gleich sein");
                    return 41;
                }
                byte[] sourcedata = new byte[3];
                byte[] targetdata = new byte[6];
                int numbytesread;
                using (FileStream sfr = File.OpenRead(sfn))
                {
                    using (StreamWriter tfw = new StreamWriter(tfn, false, Encoding.ASCII))
                    {
                        do
                        {
                            numbytesread = sfr.Read(sourcedata, 0, 3);
                            if (3 == numbytesread)
                            {
                                int bitr = 1;
                                int bits = bitr ^ (sourcedata[2] & 1);
                                int bitt = 1 ^ bits;
                                int bitu = bitt ^ ((sourcedata[2] >> 1) & 1);
                                int bitv = 1 ^ bitu;
                                int bitw = bitv ^ ((sourcedata[2] >> 2) & 1);
                                int bitx = 1 ^ bitw;
                                int bity = bitx ^ ((sourcedata[2] >> 3) & 1);
                                int bitz = 1 ^ bity;
                                int bitaa = bitz ^ ((sourcedata[2] >> 4) & 1);
                                int bitab = 1 ^ bitaa;
                                int bitac = bitab ^ ((sourcedata[2] >> 5) & 1);
                                int bitad = 1 ^ bitac;
                                int bitae = bitad ^ ((sourcedata[2] >> 6) & 1);
                                int bitaf = 1 ^ bitae;
                                int bitag = bitaf ^ ((sourcedata[2] >> 7) & 1);
                                int bitah = 1 ^ bitag;
                                int bitai = bitah ^ (sourcedata[1] & 1);
                                int bitaj = 1 ^ bitai;
                                int bitak = bitaj ^ ((sourcedata[1] >> 1) & 1);
                                int bital = 1 ^ bitak;
                                int bitam = bital ^ ((sourcedata[1] >> 2) & 1);
                                int bitan = 1 ^ bitam;
                                int bitao = bitan ^ ((sourcedata[1] >> 3) & 1);
                                int bitap = 1 ^ bitao;
                                int bitaq = bitap ^ ((sourcedata[1] >> 4) & 1);
                                int bitar = 1 ^ bitaq;
                                int bitas = bitar ^ ((sourcedata[1] >> 5) & 1);
                                int bitat = 1 ^ bitas;
                                int bitau = bitat ^ ((sourcedata[1] >> 6) & 1);
                                int bitav = 1 ^ bitau;
                                int bitaw = bitav ^ ((sourcedata[1] >> 7) & 1);
                                int bitax = 1 ^ bitaw;
                                int bitay = bitax ^ (sourcedata[0] & 1);
                                int bitaz = 1 ^ bitay;
                                int bitba = bitaz ^ ((sourcedata[0] >> 1) & 1);
                                int bitbb = 1 ^ bitba;
                                int bitbc = bitbb ^ ((sourcedata[0] >> 2) & 1);
                                int bitbd = 1 ^ bitbc;
                                int bitbe = bitbd ^ ((sourcedata[0] >> 3) & 1);
                                int bitbf = 1 ^ bitbe;
                                int bitbg = bitbf ^ ((sourcedata[0] >> 4) & 1);
                                int bitbh = 1 ^ bitbg;
                                int bitbi = bitbh ^ ((sourcedata[0] >> 5) & 1);
                                int bitbj = 1 ^ bitbi;
                                int bitbk = bitbj ^ ((sourcedata[0] >> 6) & 1);
                                int bitbl = 1 ^ bitbk;
                                int bitbm = bitbl ^ ((sourcedata[0] >> 7) & 1);
                                targetdata[0] = (byte)((bity << 7) | (bitx << 6) | (bitw << 5) | (bitv << 4) | (bitu << 3) | (bitt << 2) | (bits << 1) | bitr);
                                targetdata[1] = (byte)((bitag << 7) | (bitaf << 6) | (bitae << 5) | (bitad << 4) | (bitac << 3) | (bitab << 2) | (bitaa << 1) | bitz);
                                targetdata[2] = (byte)((bitao << 7) | (bitan << 6) | (bitam << 5) | (bital << 4) | (bitak << 3) | (bitaj << 2) | (bitai << 1) | bitah);
                                targetdata[3] = (byte)((bitaw << 7) | (bitav << 6) | (bitau << 5) | (bitat << 4) | (bitas << 3) | (bitar << 2) | (bitaq << 1) | bitap);
                                targetdata[4] = (byte)((bitbe << 7) | (bitbd << 6) | (bitbc << 5) | (bitbb << 4) | (bitba << 3) | (bitaz << 2) | (bitay << 1) | bitax);
                                targetdata[5] = (byte)((bitbm << 7) | (bitbl << 6) | (bitbk << 5) | (bitbj << 4) | (bitbi << 3) | (bitbh << 2) | (bitbg << 1) | bitbf);
                                for (int i = 0; i < 6; i++)
                                {
                                    if (0 == i)
                                    {
                                        tfw.Write(" .db ");
                                    }
                                    else
                                    {
                                        tfw.Write(", ");
                                    }
                                    tfw.Write(String.Format("0x{0:X2}", targetdata[i]));
                                }
                                tfw.WriteLine();
                            }
                            else if (0 != numbytesread)
                            {
                                Console.Error.WriteLine("Anzahl Bytes in Quelldatei nicht ohne Rest durch 3 teilbar");
                                return 1;
                            }
                        } while (3==numbytesread);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.Error.WriteLine("Fehler : " + ex.Message);
                return 40;
            }
            return 0;
        }
    }
}
